import { PartialStateUpdate } from "../state"; // TODO do not run state on content scripts
import { Actions } from '../background/message-handler';

// The actual information we care about
interface ApiInformation {
    graphQLUrl: string;
    apikey: string;
    locale: string;
}

interface RegularWindowProperties {
    add_deviceprint(): string
    LIPageData: ApiInformation;
}

interface MegaLIData {
    megaHeader: ApiInformation;
}

interface MegaWindowProperties {
    LIPageData: MegaLIData;
    add_deviceprint(): string;
}

interface LIWindowRegular extends Window {
    wrappedJSObject: RegularWindowProperties;
}

interface LIWindowMega extends Window {
    wrappedJSObject: MegaWindowProperties;
}

type LIWindow = LIWindowRegular | LIWindowMega;
type AnyWindow = Window | LIWindowRegular | LIWindowMega

function isApiInformation(liData: ApiInformation) {
    return liData.graphQLUrl !== undefined &&
        liData.apikey !== undefined &&
        liData.locale !== undefined;
}
function isRegularWindow(win: AnyWindow): win is LIWindowRegular {
    const liWindow = win as LIWindowRegular;
    return typeof liWindow.wrappedJSObject.add_deviceprint === 'function' &&
        liWindow.wrappedJSObject.LIPageData !== undefined &&
        isApiInformation(liWindow.wrappedJSObject.LIPageData);
}

function isMegaHeaderWindow(win: AnyWindow): win is LIWindowRegular {
    const liWindow = win as LIWindowMega;
    return typeof liWindow.wrappedJSObject.add_deviceprint === 'function' &&
        liWindow.wrappedJSObject.LIPageData !== undefined &&
        isApiInformation(liWindow.wrappedJSObject.LIPageData.megaHeader);
}

// Type guard that coerces the global window object to the specialized
// Landsbankinn window object with useful info attached to it. There
// are at least two different ways this data is presented.
function isLandsbankinnWindow(win: AnyWindow): win is LIWindow {
    return isRegularWindow(win) || isMegaHeaderWindow(win);
}

function extractStateUpdate(win: LIWindow): PartialStateUpdate | undefined {
    if (isRegularWindow(win)) {
        return {
            apiKey: win.wrappedJSObject.LIPageData.apikey,
            fingerprintValue: win.wrappedJSObject.add_deviceprint()
        };
    } else if (isMegaHeaderWindow(win)) {
        return {
            apiKey: win.wrappedJSObject.LIPageData.megaHeader.apikey,
            fingerprintValue: win.wrappedJSObject.add_deviceprint()
        };
    } else {
        return undefined;
    }
}

// Communicates data about the GraphQL URL and API key to the
// extension.
document.addEventListener('DOMContentLoaded', async () => {
    console.log(window);
    if (isLandsbankinnWindow(window)) {
        console.info('Ensuring extension state');
        const clientState = extractStateUpdate(window);
        await browser.runtime.sendMessage({
            action: Actions.EnsureState, clientState
        });
    }
});

export { }
