import { setDefaultOptions, format } from 'date-fns';
import { enUS } from 'date-fns/locale';
import { RawTransaction, AuthSession } from './models';

setDefaultOptions({ locale: enUS });

export enum TransactionFormat { JSON };
export enum OutputFormat { CSV };

export interface DownloadTransactionsRequest {
    accountNumber: string;
    dateFrom: Date;
    dateTo: Date;
    outputFormat: OutputFormat;
}

const formatDate = (date: Date) => format(date, 'ccc LLL dd yyyy');

function getUrl(accountNumber: string, dateFrom: Date, dateTo: Date) {
    const url = `https://netbanki.landsbankinn.is/services/accounts/${accountNumber}/transactions/`;
    const from = encodeURIComponent(formatDate(dateFrom));
    const to = encodeURIComponent(formatDate(dateTo));
    const params = `?from=${from}&to=${to}`
    return url + params;
}

export async function downloadTransactions(req: DownloadTransactionsRequest): Promise<Array<RawTransaction>> {
    const url = getUrl(req.accountNumber, req.dateFrom, req.dateTo);
    const options: RequestInit = {
        method: "GET",
        credentials: "include",
        cache: "no-cache",
        referrerPolicy: "strict-origin-when-cross-origin",
        headers: {
            "Accept": "application/json",
            "Content-Type": "application/json",
        },
    };

    const result = await fetch(url, options);
    if (result.ok) {
        return await result.json();
    } else {
        //attempt to read error json.
        const error = await result.json();
        if (error.hasOwnProperty('message')) {
            console.error(error['message']);
        }
        console.error(result);
        return new Array();
    }
}

export async function authSession(): Promise<AuthSession> {
    const url = 'https://netbanki.landsbankinn.is/api/auth/session';
    const options: RequestInit = {
        method: "GET",
        credentials: "include",
        cache: "no-cache",
        referrerPolicy: "strict-origin-when-cross-origin",
        headers: {
            "Accept": "*/*",
            "Content-Type": "application/json",
        },
    };

    const result = await fetch(url, options);
    if (result.ok) {
        return await result.json();
    } else {
        //attempt to read error json.
        const error = await result.json();
        if (error.hasOwnProperty('message')) {
            console.error(error['message']);
            return Promise.reject(error);
        } else {
            return Promise.reject(result);
        }
    }
}
